@@ -1,9 +1,9 @@
 /*
- *  $Id: dlg_keys.c,v 1.41 2016/08/28 19:36:18 tom Exp $
+ *  $Id: dlg_keys.c,v 1.48 2019/07/25 00:41:10 tom Exp $
  *
  *  dlg_keys.c -- runtime binding support for dialog
  *
- *  Copyright 2006-2014,2016 Thomas E. Dickey
+ *  Copyright 2006-2018,2019 Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -61,10 +61,11 @@ dlg_register_window(WINDOW *win, const char *name, DLG_KEYS_BINDING * binding)
 	p->win = win;
 	p->name = name;
 	p->binding = binding;
-	if (q != 0)
+	if (q != 0) {
 	    q->link = p;
-	else
+	} else {
 	    all_bindings = p;
+	}
     }
 #if defined(HAVE_DLG_TRACE) && defined(HAVE_RC_FILE)
     /*
@@ -74,8 +75,10 @@ dlg_register_window(WINDOW *win, const char *name, DLG_KEYS_BINDING * binding)
      * registered, there is no other way to see what bindings are available,
      * than by running dialog and tracing it.
      */
-    dlg_trace_msg("# dlg_register_window %s\n", name);
+    DLG_TRACE(("# dlg_register_window %s\n", name));
+    dlg_dump_keys(dialog_state.trace_output);
     dlg_dump_window_keys(dialog_state.trace_output, win);
+    DLG_TRACE(("# ...done dlg_register_window %s\n", name));
 #endif
 }
 
@@ -198,8 +201,8 @@ dlg_unregister_window(WINDOW *win)
  * Parameters:
  *	win is the window on which the wgetch() was done.
  *	curses_key is the value returned by wgetch().
- *	fkey in/out (on input, it is true if curses_key is a function key,
- *		and on output, it is true if the result is a function key).
+ *	fkey in/out (on input, it is nonzero if curses_key is a function key,
+ *		and on output, it is nonzero if the result is a function key).
  */
 int
 dlg_lookup_key(WINDOW *win, int curses_key, int *fkey)
@@ -271,6 +274,17 @@ dlg_result_key(int dialog_key, int fkey GCC_UNUSED, int *resultp)
 {
     int done = FALSE;
 
+#ifdef KEY_RESIZE
+    if (dialog_state.had_resize) {
+	if (dialog_key == ERR) {
+	    dialog_key = 0;
+	} else {
+	    dialog_state.had_resize = FALSE;
+	}
+    } else if (fkey && dialog_key == KEY_RESIZE) {
+	dialog_state.had_resize = TRUE;
+    }
+#endif
 #ifdef HAVE_RC_FILE
     if (fkey) {
 	switch ((DLG_KEYS_ENUM) dialog_key) {
@@ -560,13 +574,13 @@ make_binding(char *widget, int curses_key, int is_function, int dialog_key)
     LIST_BINDINGS *entry = 0;
     DLG_KEYS_BINDING *data = 0;
     char *name;
-    LIST_BINDINGS *p, *q;
     DLG_KEYS_BINDING *result = find_binding(widget, curses_key);
 
     if (result == 0
 	&& (entry = dlg_calloc(LIST_BINDINGS, 1)) != 0
 	&& (data = dlg_calloc(DLG_KEYS_BINDING, 2)) != 0
 	&& (name = dlg_strclone(widget)) != 0) {
+	LIST_BINDINGS *p, *q;
 
 	entry->name = name;
 	entry->binding = data;
@@ -603,7 +617,6 @@ make_binding(char *widget, int curses_key, int is_function, int dialog_key)
 static int
 decode_escaped(char **string)
 {
-    unsigned n;
     int result = 0;
 
     if (IsOctal(**string)) {
@@ -614,6 +627,8 @@ decode_escaped(char **string)
 	    result = (result << 3) | (ch - '0');
 	}
     } else {
+	unsigned n;
+
 	for (n = 0; n < TableSize(escaped_letters); ++n) {
 	    if (**string == escaped_letters[n].letter) {
 		*string += 1;
@@ -660,13 +675,8 @@ int
 dlg_parse_bindkey(char *params)
 {
     char *p = skip_white(params);
-    char *q;
-    bool escaped = FALSE;
-    int modified = 0;
     int result = FALSE;
-    unsigned xx;
     char *widget;
-    int is_function = FALSE;
     int curses_key;
     int dialog_key;
 
@@ -676,6 +686,12 @@ dlg_parse_bindkey(char *params)
 
     p = skip_black(p);
     if (p != widget && *p != '\0') {
+	char *q;
+	unsigned xx;
+	bool escaped = FALSE;
+	int modified = 0;
+	int is_function = FALSE;
+
 	*p++ = '\0';
 	p = skip_white(p);
 	q = p;
@@ -712,7 +728,7 @@ dlg_parse_bindkey(char *params)
 		char fprefix[2];
 		char check[2];
 		int keynumber;
-		if (sscanf(q, "%[Ff]%d%c", fprefix, &keynumber, check) == 2) {
+		if (sscanf(q, "%1[Ff]%d%c", fprefix, &keynumber, check) == 2) {
 		    curses_key = KEY_F(keynumber);
 		    is_function = TRUE;
 		} else {
@@ -807,12 +823,28 @@ dump_dialog_key(FILE *fp, int dialog_key)
 }
 
 static void
-dump_one_binding(FILE *fp, const char *widget, DLG_KEYS_BINDING * binding)
+dump_one_binding(FILE *fp,
+		 WINDOW *win,
+		 const char *widget,
+		 DLG_KEYS_BINDING * binding)
 {
+    int actual;
+    int fkey = (binding->curses_key > 255);
+
     fprintf(fp, "bindkey %s ", widget);
     dump_curses_key(fp, binding->curses_key);
     fputc(' ', fp);
     dump_dialog_key(fp, binding->dialog_key);
+    actual = dlg_lookup_key(win, binding->curses_key, &fkey);
+#ifdef KEY_MOUSE
+    if (is_DLGK_MOUSE(binding->curses_key) && is_DLGK_MOUSE(actual)) {
+	;			/* EMPTY */
+    } else
+#endif
+    if (actual != binding->dialog_key) {
+	fprintf(fp, "\t# overridden by ");
+	dump_dialog_key(fp, actual);
+    }
     fputc('\n', fp);
 }
 
@@ -832,12 +864,13 @@ dlg_dump_window_keys(FILE *fp, WINDOW *win)
 	for (p = all_bindings; p != 0; p = p->link) {
 	    if (p->win == win) {
 		if (dlg_strcmp(last, p->name)) {
-		    fprintf(fp, "\n# key bindings for %s widgets\n",
-			    !strcmp(p->name, WILDNAME) ? "all" : p->name);
+		    fprintf(fp, "# key bindings for %s widgets%s\n",
+			    !strcmp(p->name, WILDNAME) ? "all" : p->name,
+			    win == 0 ? " (user-defined)" : "");
 		    last = p->name;
 		}
 		for (q = p->binding; q->is_function_key >= 0; ++q) {
-		    dump_one_binding(fp, p->name, q);
+		    dump_one_binding(fp, win, p->name, q);
 		}
 	    }
 	}
