@@ -1,9 +1,9 @@
 /*
- *  $Id: textbox.c,v 1.107 2012/07/01 18:13:24 Zoltan.Kelemen Exp $
+ *  $Id: textbox.c,v 1.118 2019/07/25 00:07:15 tom Exp $
  *
  *  textbox.c -- implements the text box
  *
- *  Copyright 2000-2011,2012	Thomas E.  Dickey
+ *  Copyright 2000-2018,2019	Thomas E.  Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -55,6 +55,7 @@ lseek_obj(MY_OBJ * obj, long offset, int mode)
     long fpos;
     if ((fpos = (long) lseek(obj->fd, (off_t) offset, mode)) == -1) {
 	switch (mode) {
+	default:
 	case SEEK_CUR:
 	    dlg_exiterr("Cannot get file position");
 	    break;
@@ -75,6 +76,37 @@ ftell_obj(MY_OBJ * obj)
     return lseek_obj(obj, 0L, SEEK_CUR);
 }
 
+static void
+lseek_set(MY_OBJ * obj, long offset)
+{
+    long actual = lseek_obj(obj, offset, SEEK_SET);
+
+    if (actual != offset) {
+	dlg_exiterr("Cannot set file position to %ld (actual %ld)\n",
+		    offset, actual);
+    }
+}
+
+static void
+lseek_end(MY_OBJ * obj, long offset)
+{
+    long actual = lseek_obj(obj, offset, SEEK_END);
+
+    if (offset == 0L && actual > offset) {
+	obj->file_size = actual;
+    }
+}
+
+static void
+lseek_cur(MY_OBJ * obj, long offset)
+{
+    long actual = lseek_obj(obj, offset, SEEK_CUR);
+
+    if (actual != offset) {
+	DLG_TRACE(("# Lseek returned %ld, expected %ld\n", actual, offset));
+    }
+}
+
 static char *
 xalloc(size_t size)
 {
@@ -95,14 +127,14 @@ xalloc(size_t size)
 static void
 read_high(MY_OBJ * obj, size_t size_read)
 {
-    char *buftab, ch;
-    int i = 0, j, n, tmpint;
-    long begin_line;
+    char *buftab;
 
     /* Allocate space for read buffer */
     buftab = xalloc(size_read + 1);
 
     if ((obj->fd_bytes_read = read(obj->fd, buftab, size_read)) != -1) {
+	int i = 0, j, n, tmpint;
+	long begin_line;
 
 	buftab[obj->fd_bytes_read] = '\0';	/* mark end of valid data */
 
@@ -147,7 +179,9 @@ read_high(MY_OBJ * obj, size_t size_read)
 
 	j = 0;
 	begin_line = 0;
-	while (j < obj->fd_bytes_read)
+	while (j < obj->fd_bytes_read) {
+	    char ch;
+
 	    if (((ch = buftab[j++]) == TAB) && (dialog_vars.tab_correct != 0)) {
 		tmpint = (dialog_state.tab_len
 			  - ((int) ((long) i - begin_line) % dialog_state.tab_len));
@@ -158,6 +192,7 @@ read_high(MY_OBJ * obj, size_t size_read)
 		    begin_line = i + 1;
 		obj->buf[i++] = ch;
 	    }
+	}
 
 	obj->buf[i] = '\0';	/* mark end of valid data */
 
@@ -197,7 +232,7 @@ tabize(MY_OBJ * obj, long val, long *first_pos)
 
     fpos = ftell_obj(obj);
 
-    lseek_obj(obj, fpos - obj->fd_bytes_read, SEEK_SET);
+    lseek_set(obj, fpos - obj->fd_bytes_read);
 
     /* Allocate space for read buffer */
     buftab = xalloc((size_t) val + 1);
@@ -224,7 +259,7 @@ tabize(MY_OBJ * obj, long val, long *first_pos)
 	    count++;
     }
 
-    lseek_obj(obj, fpos, SEEK_SET);
+    lseek_set(obj, fpos);
     free(buftab);
     return count;
 }
@@ -309,10 +344,10 @@ back_lines(MY_OBJ * obj, long n)
 		/* Really possible to move backward BUF_SIZE/2 bytes? */
 		if (fpos < BUF_SIZE / 2 + obj->fd_bytes_read) {
 		    /* No, move less than */
-		    lseek_obj(obj, 0L, SEEK_SET);
+		    lseek_set(obj, 0L);
 		    val_to_tabize = fpos - obj->fd_bytes_read;
 		} else {	/* Move backward BUF_SIZE/2 bytes */
-		    lseek_obj(obj, -(BUF_SIZE / 2 + obj->fd_bytes_read), SEEK_CUR);
+		    lseek_cur(obj, -(BUF_SIZE / 2 + obj->fd_bytes_read));
 		    val_to_tabize = BUF_SIZE / 2;
 		}
 		read_high(obj, BUF_SIZE);
@@ -340,10 +375,10 @@ back_lines(MY_OBJ * obj, long n)
 		    /* Really possible to move backward BUF_SIZE/2 bytes? */
 		    if (fpos < BUF_SIZE / 2 + obj->fd_bytes_read) {
 			/* No, move less than */
-			lseek_obj(obj, 0L, SEEK_SET);
+			lseek_set(obj, 0L);
 			val_to_tabize = fpos - obj->fd_bytes_read;
 		    } else {	/* Move backward BUF_SIZE/2 bytes */
-			lseek_obj(obj, -(BUF_SIZE / 2 + obj->fd_bytes_read), SEEK_CUR);
+			lseek_cur(obj, -(BUF_SIZE / 2 + obj->fd_bytes_read));
 			val_to_tabize = BUF_SIZE / 2;
 		    }
 		    read_high(obj, BUF_SIZE);
@@ -493,7 +528,7 @@ get_search_term(WINDOW *dialog, char *input, int height, int width)
 		  searchbox_attr,
 		  searchbox_border_attr,
 		  searchbox_border2_attr);
-    wattrset(widget, searchbox_title_attr);
+    dlg_attrset(widget, searchbox_title_attr);
     (void) wmove(widget, 0, (box_width - len_caption) / 2);
 
     indx = dlg_index_wchars(caption);
@@ -539,16 +574,17 @@ static bool
 perform_search(MY_OBJ * obj, int height, int width, int key, char *search_term)
 {
     int dir;
-    long tempinx;
-    long fpos;
-    int result;
-    bool found;
-    bool temp, temp1;
     bool moved = FALSE;
 
     /* set search direction */
     dir = (key == '/' || key == 'n') ? 1 : 0;
     if (dir ? !obj->end_reached : !obj->begin_reached) {
+	long tempinx;
+	long fpos;
+	int result;
+	bool found;
+	bool temp, temp1;
+
 	if (key == 'n' || key == 'N') {
 	    if (search_term[0] == '\0') {	/* No search term yet */
 		(void) beep();
@@ -567,7 +603,7 @@ perform_search(MY_OBJ * obj, int height, int width, int key, char *search_term)
 	    }
 #endif
 	    /* ESC pressed, or no search term, reprint page to clear box */
-	    wattrset(obj->text, dialog_attr);
+	    dlg_attrset(obj->text, dialog_attr);
 	    back_lines(obj, obj->page_length);
 	    return TRUE;
 	}
@@ -596,7 +632,7 @@ perform_search(MY_OBJ * obj, int height, int width, int key, char *search_term)
 	if (found == FALSE) {	/* not found */
 	    (void) beep();
 	    /* Restore program state to that before searching */
-	    lseek_obj(obj, fpos, SEEK_SET);
+	    lseek_set(obj, fpos);
 
 	    read_high(obj, BUF_SIZE);
 
@@ -612,7 +648,7 @@ perform_search(MY_OBJ * obj, int height, int width, int key, char *search_term)
 	    back_lines(obj, 1L);
 	}
 	/* Reprint page */
-	wattrset(obj->text, dialog_attr);
+	dlg_attrset(obj->text, dialog_attr);
 	moved = TRUE;
     } else {			/* no need to find */
 	(void) beep();
@@ -624,7 +660,7 @@ perform_search(MY_OBJ * obj, int height, int width, int key, char *search_term)
  * Display text from a file in a dialog box.
  */
 int
-dialog_textbox(const char *title, const char *file, int height, int width)
+dialog_textbox(const char *title, const char *filename, int height, int width)
 {
     /* *INDENT-OFF* */
     static DLG_KEYS_BINDING binding[] = {
@@ -647,7 +683,7 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 	DLG_KEYS_DATA( DLGK_PAGE_LAST,  'G' ),
 	DLG_KEYS_DATA( DLGK_PAGE_LAST,  KEY_END ),
 	DLG_KEYS_DATA( DLGK_PAGE_LAST,  KEY_LL ),
-	DLG_KEYS_DATA( DLGK_PAGE_NEXT,  ' ' ),
+	DLG_KEYS_DATA( DLGK_PAGE_NEXT,  CHR_SPACE ),
 	DLG_KEYS_DATA( DLGK_PAGE_NEXT,  KEY_NPAGE ),
 	DLG_KEYS_DATA( DLGK_PAGE_PREV,  'B' ),
 	DLG_KEYS_DATA( DLGK_PAGE_PREV,  'b' ),
@@ -677,6 +713,12 @@ dialog_textbox(const char *title, const char *file, int height, int width)
     int button = dlg_default_button();
     int min_width = 12;
 
+    DLG_TRACE(("# textbox args:\n"));
+    DLG_TRACE2S("title", title);
+    DLG_TRACE2S("filename", filename);
+    DLG_TRACE2N("height", height);
+    DLG_TRACE2N("width", width);
+
     search_term[0] = '\0';	/* no search term entered yet */
 
     memset(&obj, 0, sizeof(obj));
@@ -687,15 +729,15 @@ dialog_textbox(const char *title, const char *file, int height, int width)
     obj.buttons = dlg_exit_label();
 
     /* Open input file for reading */
-    if ((obj.fd = open(file, O_RDONLY)) == -1)
-	dlg_exiterr("Can't open input file %s", file);
+    if ((obj.fd = open(filename, O_RDONLY)) == -1)
+	dlg_exiterr("Can't open input file %s", filename);
 
     /* Get file size. Actually, 'file_size' is the real file size - 1,
        since it's only the last byte offset from the beginning */
-    obj.file_size = lseek_obj(&obj, 0L, SEEK_END);
+    lseek_end(&obj, 0L);
 
     /* Restore file pointer to beginning of file after getting file size */
-    lseek_obj(&obj, 0L, SEEK_SET);
+    lseek_set(&obj, 0L);
 
     read_high(&obj, BUF_SIZE);
 
@@ -706,7 +748,7 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 #endif
     moved = TRUE;
 
-    dlg_auto_sizefile(title, file, &height, &width, 2, min_width);
+    dlg_auto_sizefile(title, filename, &height, &width, 2, min_width);
     dlg_print_size(height, width);
     dlg_ctl_size(height, width);
 
@@ -833,7 +875,7 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 
 		    if (fpos > obj.fd_bytes_read) {
 			/* Yes, we have to read it in */
-			lseek_obj(&obj, 0L, SEEK_SET);
+			lseek_set(&obj, 0L);
 
 			read_high(&obj, BUF_SIZE);
 		    }
@@ -848,7 +890,7 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 
 		if (fpos < obj.file_size) {
 		    /* Yes, we have to read it in */
-		    lseek_obj(&obj, -BUF_SIZE, SEEK_END);
+		    lseek_end(&obj, -BUF_SIZE);
 
 		    read_high(&obj, BUF_SIZE);
 		}
@@ -910,6 +952,7 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 		break;
 #ifdef KEY_RESIZE
 	    case KEY_RESIZE:
+		dlg_will_resize(dialog);
 		/* reset data */
 		height = old_height;
 		width = old_width;
@@ -917,7 +960,6 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 		/* repaint */
 		dlg_clear();
 		dlg_del_window(dialog);
-		refresh();
 		dlg_mouse_free_regions();
 		goto retry;
 #endif
